@@ -9,45 +9,40 @@
 """
 
 import argparse
-try:
-    import configparser
-except ImportError:
-    import ConfigParser as configparser
+import codecs
+import configparser
 import logging
 import os
-import sys
 import re
+import sys
 
 from collections import defaultdict
+from collections import OrderedDict
 from datetime import datetime
 
 import docutils
 from docutils.frontend import OptionParser
-from docutils.utils import new_document
 from docutils.parsers.rst import Parser
 from docutils.writers.html4css1 import HTMLTranslator
 
-from jinja2 import Environment, PackageLoader
-
+from jinja2 import Environment
+from jinja2 import PackageLoader
+import locale
 #
 # globals
 #
 config = None
-
-# a dict that keeps entries keyed by month.
-# later, we can walk each key kept in all_months
-# to build the pages
-all_entries = defaultdict(list)
-
-# set of keys for all_entries.  sorted by parse_entries into reverse
-# date order (i.e. latest month is first)
-all_months = set()
+locale.setlocale(locale.LC_TIME, '')
 
 
-class Entry():
+class Todo(object):
+    """The todo list"""
+    def __init__(self, body_html):
+        self.body_html = re.sub(r"<h1>.*</h1>", '', body_html)
 
+
+class Entry(object):
     """Representation of a single day's entry in the RST file"""
-
     def __init__(self, date, body):
         self.date = date
         self.body = body
@@ -67,14 +62,34 @@ class Entry():
         return str(self.date)
 
 
-# go through the input_file and pull out each section; parse the date
-# and create an Entry() object.  Entries go into all_entries keyed by
-# the month it was created in; each month we see gets an entry in
-# all_months
 def parse_entries(input_file):
-    global all_months, all_entries
+    """Parse entries from input file
 
-    file = open(input_file)
+    Go through the input_file and pull out each section; parse the date
+    and create an Entry() object.  Entries go into all_entries keyed by
+    the month it was created in; each month we see gets an entry in
+    all_months
+
+    Returns a dict with three keys:
+
+     - all_entries : dict keyed by month with list of Entry objects
+     - all_months : set of all months in reverse date order
+             (i.e. latest month first)
+     - todo : the todo list section, or None if not available
+    """
+
+    # a dict that keeps entries keyed by month.  later, we can walk
+    # each key kept in all_months to build the pages
+    all_entries = defaultdict(list)
+
+    # set of keys for all_entries.  sorted by parse_entries into
+    # reverse date order (i.e. latest month is first)
+    all_months = set()
+
+    # the todo list section
+    todo = None
+
+    file = codecs.open(input_file, 'r', 'utf-8')
     try:
         text = file.read()
     finally:
@@ -88,6 +103,17 @@ def parse_entries(input_file):
     parser.parse(text, docroot)
 
     for i in docroot.traverse(condition=docutils.nodes.section):
+        try:
+            if str(i.children[0]) == "<title>todo</title>":
+                logging.debug("Found todo section")
+                translator = HTMLTranslator(docroot)
+                i.walkabout(translator)
+                body = ''.join(translator.body)
+                todo = Todo(body)
+                continue
+        except IndexError:
+            pass
+
         try:
             date_string = re.findall(r'(\d{4}-\d{1,2}-\d{1,2})',
                                      str(i.children[0]))[0]
@@ -109,13 +135,33 @@ def parse_entries(input_file):
 
     all_months = sorted(all_months, reverse=True)
 
+    return {'all_months': all_months,
+            'all_entries': all_entries,
+            'todo': todo}
+
+
+def write_html(all_months, all_entries, todo):
+    """Write out HTML pages.
+
+    Takes arguments from parse_entries
+
+    Arguments:
+    :param all_months:
+    :param all_entries:
+    :param todo:
+    """
 
-# write out the HTML pages via our template
-def write_html():
     env = Environment(loader=PackageLoader('rstdiary', 'templates'))
     template = env.get_template('page.html')
     output_dir = config.get('rstdiary', 'output_dir')
 
+    # since this is in order, we can bunch them by year to get
+    # an accordian list in the output
+    all_months_by_year = OrderedDict()
+    for month in all_months:
+        y, m = month.split("-")
+        all_months_by_year.setdefault(y, []).append(m)
+
     first = True
     for index, month in enumerate(all_months):
         month_entries = all_entries[month]
@@ -123,14 +169,18 @@ def write_html():
 
         string_month = datetime.strptime(month, "%Y-%m").strftime("%B %Y")
 
+        previous_month = all_months[index - 1] if index >= 1 else None
+        next_month = (all_months[index + 1]
+                      if index < len(all_months) - 1 else None)
+
         output = template.render(title=config.get('rstdiary', 'title'),
                                  about=config.get('rstdiary', 'about'),
                                  month=string_month,
-                                 all_months=all_months,
+                                 all_months_by_year=all_months_by_year,
                                  month_entries=month_entries,
-                                 previous_month=
-                                 all_months[index - 1] if index >= 1 else None,
-                                 next_month=all_months[index + 1] if index < len(all_months)-1 else None)
+                                 previous_month=previous_month,
+                                 next_month=next_month,
+                                 todo=todo)
 
         filename = os.path.join(output_dir, '%s.html' % month)
 
@@ -145,12 +195,22 @@ def write_html():
             first = False
 
         logging.debug("Writing %s" % filename)
-        with open(filename, 'w') as f:
+        with codecs.open(filename, 'w', 'utf-8') as f:
             f.write(output)
 
 
 # a simple atom feed of just the latest month
-def write_atom():
+def write_atom(all_months, all_entries, todo):
+    """Write out ATOM feed
+
+    Only includes the latest month.  Takes arguments from
+    parse_entries.
+
+    Arguments:
+    :param all_months:
+    :param all_entries:
+    :param todo:
+    """
     env = Environment(loader=PackageLoader('rstdiary', 'templates'))
     template = env.get_template('atom.xml')
     output_dir = config.get('rstdiary', 'output_dir')
@@ -169,7 +229,7 @@ def write_atom():
 
     filename = os.path.join(output_dir, 'atom.xml')
     logging.debug("Writing atom feed for %s to %s" % (month, filename))
-    with open(filename, 'w') as f:
+    with codecs.open(filename, 'w', 'utf-8') as f:
         f.write(output)
 
 
@@ -203,7 +263,8 @@ def main():
         sys.stderr.write("Config file doesn't contain section [rstdiary]\n")
         sys.exit(1)
 
-    if not os.path.isdir(config.get('rstdiary', 'output_dir')):
+    output_dir = config.get('rstdiary', 'output_dir')
+    if not os.path.isdir(output_dir):
         sys.stderr.write("output_dir <%s>: not a directory\n" % output_dir)
         sys.exit(1)
 
@@ -213,10 +274,11 @@ def main():
         sys.stderr.write("Invalid input: %s\n" % input_filename)
         sys.exit(1)
 
-    parse_entries(input_filename)
+    results = parse_entries(input_filename)
+
+    write_html(**results)
+    write_atom(**results)
 
-    write_html()
-    write_atom()
 
 if __name__ == "__main__":
     main()
